@@ -134,6 +134,9 @@ static int StripBareCR = 0;
 /* Allow new connections to queue? */
 static int AllowNewConnectionsToQueue = 0;
 
+/* Is it OK to call smfi_setsymlist? */
+static int setsymlist_ok = 0;
+
 /* Run as this user */
 static char *user = NULL;
 extern int drop_privs(char const *user, uid_t uid, gid_t gid);
@@ -163,7 +166,7 @@ static char *StandardSendmailMacros[] = {
     "cert_issuer", "cert_subject", "cipher", "cipher_bits", "daemon_name",
     "daemon_port",
     "i", "if_addr", "if_name", "j", "mail_addr", "mail_host", "mail_mailer",
-    "tls_version", "verify",
+    "tls_version", "verify", "rcpt_addr", "rcpt_host", "rcpt_mailer",
     /* End of macros MUST be marked with NULL! */
     NULL
 };
@@ -523,7 +526,7 @@ mfconnect(SMFICTX *ctx, char *hostname, _SOCK_ADDR *sa)
     if (!AllowNewConnectionsToQueue) {
 	int is_local;
 	int required_slaves;
-	int n = MXCheckFreeSlaves(MultiplexorSocketName);
+	int n = MXCheckFreeSlaves(MultiplexorSocketName, NULL);
 	if (n < 0) {
 	    syslog(LOG_WARNING, "mfconnect: Error communicating with multiplexor");
 	    DEBUG_EXIT("mfconnect", "SMFIS_TEMPFAIL");
@@ -900,7 +903,7 @@ envfrom(SMFICTX *ctx, char **from)
     if (data->dir) {
 	/* Clean data->dir up if it's still lying around */
 	if (access(data->dir, R_OK) == 0) {
-	    (void) rm_r(ctx, data->dir);
+	    (void) rm_r(data->qid, data->dir);
 	}
 	free(data->dir);
 	data->dir = NULL;
@@ -1117,6 +1120,13 @@ static sfsistat mf_negotiate(SMFICTX *ctx,
     /* Don't want leading spaces */
     *pf1 &= (~SMFIP_HDR_LEADSPC);
 
+    /*** libmilter 8.14.3 leaked memory, so don't use smfi_setsymlist
+	 unless invoked with -y option ***/
+
+    if (!setsymlist_ok) {
+	return SMFIS_CONTINUE;
+    }
+
     /* Send along the list of macros we want */
     dbuf_init(&dbuf);
 
@@ -2230,6 +2240,7 @@ usage(void)
     fprintf(stderr, "Options:\n");
     fprintf(stderr, "  -h                -- Print usage info and exit\n");
     fprintf(stderr, "  -v                -- Print version and exit\n");
+    fprintf(stderr, "  -y                -- Invoke smfi_setsymlist to set macro list\n");
     fprintf(stderr, "  -m /path          -- Use /path as UNIX-domain socket for multiplexor\n");
     fprintf(stderr, "  -p /path          -- Path to UNIX-domain socket for sendmail communication\n");
     fprintf(stderr, "  -U user           -- Run as user instead of root\n");
@@ -2342,8 +2353,12 @@ main(int argc, char **argv)
     }
 
     /* Process command line options */
-    while ((c = getopt(argc, argv, "GNCDHL:MP:R:S:TU:Xa:b:cdhkm:p:qrstvx:z:")) != -1) {
+    while ((c = getopt(argc, argv, "GNCDHL:MP:R:S:TU:Xa:b:cdhkm:p:qrstvx:z:y")) != -1) {
 	switch (c) {
+	case 'y':
+	    setsymlist_ok = 1;
+	    break;
+
 	case 'G':
 	    socket_umask = 007;
 	    file_umask   = 027;
@@ -2709,7 +2724,7 @@ main(int argc, char **argv)
 	struct timeval sleeptime;
 	sleeptime.tv_sec = 0;
 	sleeptime.tv_usec = 200000;
-	if (MXCheckFreeSlaves(MultiplexorSocketName) >= 0) {
+	if (MXCheckFreeSlaves(MultiplexorSocketName, NULL) >= 0) {
 	    mx_alive = 1;
 	    break;
 	}
@@ -2805,7 +2820,7 @@ remove_working_directory(SMFICTX *ctx, struct privdata *data)
 	return;
     }
 
-    if (rm_r(ctx, data->dir) < 0) {
+    if (rm_r(data->qid, data->dir) < 0) {
 	syslog(LOG_ERR, "%s: failed to clean up %s: %m",
 	       data->qid, data->dir);
     }
